def check(ht,cv2,np,Image,hands,img,hand,heartImg,heartLeftImg,heartRightImg):
    heart = half_heart_left = half_heart_right = False
    for i in hands:
        label = i["label"]
        lms = i["lms"]
        num_check = 0
        lms_index = np.array([8,12,16,20])
        if label == "Right" and "Right" not in hand:
            for i in lms_index:
                if lms[i].x > lms[i-2].x:
                    num_check+=1
            if num_check==4:
                if lms[i].y >lms[i-2].y and lms[i].y < lms[4].y and lms[i-2].y < lms[4].y:
                    if lms[i].y > lms[4].y:
                        dis = (lms[4].y*100)/lms[i].y
                    elif lms[i].y < lms[4].y:
                        dis = (lms[i].y*100)/lms[4].y   
                    if dis <= 86:
                        half_heart_right = True
        elif label == "Left" and "Left" not in hand:
            for i in lms_index:
                if lms[i].x < lms[i-2].x:
                    num_check += 1
            if num_check==4:
                if lms[i].y >lms[i-2].y and lms[i].y < lms[4].y and lms[i-2].y < lms[4].y:
                    if lms[i].y > lms[4].y:
                        dis = (lms[4].y*100)/lms[i].y
                    elif lms[i].y < lms[4].y:
                        dis = (lms[i].y*100)/lms[4].y
                    if dis <= 86:
                        half_heart_left = True
    if half_heart_right and half_heart_left:
        lms_index = np.array([4,8,12,16,20])
        lms1 = hands[0]["lms"]
        lms2 = hands[1]["lms"]
        result = 0
        for i in lms_index:
            if lms1[i].x > lms2[i].x:
                dis_x = (lms2[i].x*100)/lms1[i].x
            elif lms1[i].x < lms2[i].x:
                dis_x = (lms1[i].x*100)/lms2[i].x
            if lms1[i].y > lms2[i].y:
                dis_y = (lms2[i].y*100)/lms2[i].y
            elif lms1[i].y < lms2[i].y:
                dis_y = (lms1[i].y*100)/lms2[i].y
            if dis_x >= 90 and dis_y >= 90:
                result += 1
                
        if result == 5:
            heart = True
            
    if heart:
        heart = True
        lms1 = hands[0]["lms"][0]
        lms2 = hands[1]["lms"][0]
        lms_x = int((ht.getCoord(lms1).x+ht.getCoord(lms2).x)/2)
        lms_y = int((ht.getCoord(lms1).y+ht.getCoord(lms2).y)/2)
        img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
        img = img.copy()
        img.paste(heartImg,(lms_x-100,lms_y-175),mask=heartImg)
        img = cv2.cvtColor(np.array(img), cv2.COLOR_RGB2BGR)
        hand.append("Right")
        hand.append("Left")
        return True,img,hand

    elif half_heart_right or half_heart_left:
        heart = True
        for i in hands:
            label = i["label"]
            lms = i["lms"]
            img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
            img = img.copy()
            if label == "Left" and "Left" not in hand:
                if half_heart_left:
                    hand.append(label)
                    img.paste(heartLeftImg,(ht.getCoord(lms[4]).x-75,ht.getCoord(lms[4]).y-200),mask=heartLeftImg)
            elif label == "Right" and "Right" not in hand:
                if half_heart_right:
                    hand.append(label)
                    img.paste(heartRightImg,(ht.getCoord(lms[4]).x-225,ht.getCoord(lms[4]).y-200),mask=heartRightImg)
            img = cv2.cvtColor(np.array(img), cv2.COLOR_RGB2BGR)
        return True,img,hand


def check(cv2,np,hands,img,hand):
    result = False
    for i in hands:
        label = i["label"]
        lms = i["lms"]
        
        result=None
        num_check = 0
        lms_index = np.array([6,10,14,18])
        if label == "Right":
            for i in lms_index:
                if lms[i].y < lms[i+1].y:
                    num_check+=1
                if num_check==4:
                    if lms[4].x < lms[3].x and lms[4].y>lms[i].y:
                            result=True
                    elif lms[4].x < lms[3].x and lms[4].y<lms[i].y:
                            result=True
        if label == "Left":
            for i in lms_index:
                if lms[i].y < lms[i+1].y:
                    num_check+=1
                if num_check==4:
                    if lms[4].x > lms[3].x and lms[4].y>lms[i].y:
                        result=True
                    elif lms[4].x > lms[3].x and lms[4].y<lms[i].y:
                        result=True

        if result == True:
            if label == "Right" and "Right" not in hand:
                cv2.rectangle(img,(25,25),(350,150),(0,255,0),-1)
                cv2.putText(img,"Power",(50,120),
                            cv2.FONT_HERSHEY_SIMPLEX,3,(0,0,255),10,cv2.LINE_AA) 
                hand.append(label)

            elif label == "Left" and "Left" not in hand:
                cv2.rectangle(img,(925,25),(1250,150),(0,255,0),-1)
                cv2.putText(img,"Power",(950,120),
                            cv2.FONT_HERSHEY_SIMPLEX,3,(255,0,0),10,cv2.LINE_AA) 
                hand.append(label)
    if result:            
        return True,img,hand


import cv2
import mediapipe as mp
from collections import namedtuple

class handDetector():
    def __init__(self, mode=False, maxHand=2, modelComplex=1, detectionCon=0.5, trackCon=0.5, drawLandmark=True):
        self.mpHands = mp.solutions.hands
        self.mpDraw = mp.solutions.drawing_utils
        self.hands = self.mpHands.Hands(mode, maxHand, modelComplex, detectionCon, trackCon)
        self.drawLandmark = drawLandmark
    
    def getCoord(self,lm):
        h, w, c = self.img.shape
        coord = namedtuple('coord', ['x', 'y'])
        coord = coord(int(lm.x*w), int(lm.y*h))
        return coord
    
    def findHands(self,img):
        self.img = img
        imgRGB = cv2.cvtColor(self.img, cv2.COLOR_BGR2RGB)
        self.results = self.hands.process(imgRGB)
        hand_li = None
        if self.results.multi_hand_landmarks:
            hand_li = []
            for hand_index, handLms in enumerate(self.results.multi_hand_landmarks):
                label = self.results.multi_handedness[hand_index].classification[0].label
                if label == "Left":
                    label = "Right"
                    color = (0,0,255) #Red
                else:
                    label = "Left"
                    color = (255,0,0) #Blue
                if self.drawLandmark:
                    self.mpDraw.draw_landmarks(self.img, handLms, self.mpHands.HAND_CONNECTIONS, 
                                        # Joint Land Mark Color
                                        self.mpDraw.DrawingSpec(color=color, thickness=2, circle_radius=2),
                                        # Land Mark Line Connector Color
                                        self.mpDraw.DrawingSpec(color=color, thickness=2, circle_radius=2))
                hand = {
                    "label": label,
                    "lms": handLms.landmark
                }
                hand_li.append(hand)
        return self.img, hand_li
    




def check(ht,cv2,np,hands,img,hand):
    check_thumb = False
    for i in hands:
        label = i["label"]
        lms = i["lms"]
        result=None
        num_check = 0
        lms_index = np.array([8,12,16,20])
        if label == "Right" and "Right" not in hand:   
            for i in lms_index:
                if lms[i].x < lms[i-2].x:
                    num_check+=1
            if num_check==4:
                if lms[2].x < lms[6].x:
                    if lms[2].y >  lms[4].y:
                        if lms[5].y<lms[17].y and lms[4].x<lms[5].x:
                            result = True
                            hand.append(label)
                    if lms[2].y <  lms[4].y:
                        if lms[5].y>lms[17].y:
                            result = False
                            hand.append(label)

        if label == "Left" and "Left" not in hand:
            for i in lms_index:
                if lms[i].x > lms[i-2].x:
                    num_check+=1
            if num_check==4:
                if lms[2].x > lms[6].x:
                    if lms[2].y >  lms[4].y:
                        if lms[5].y<lms[17].y and lms[4].x > lms[5].x:
                            result = True
                            hand.append(label)
                    if lms[2].y <  lms[4].y:
                        if lms[5].y>lms[17].y:
                            result = False
                            hand.append(label)
        # print(result)
        if result == True:
            if label == "Right":
                cv2.rectangle(img,(25,25),(550,150),(0,255,0),-1)
                cv2.putText(img,"Good Job!",(50,120),
                            cv2.FONT_HERSHEY_SIMPLEX,3,(0,0,255),10,cv2.LINE_AA) 
                hand.append(label)
            elif label == "Left":
                cv2.rectangle(img,(725,25),(1250,150),(0,255,0),-1)
                cv2.putText(img,"Good Job!",(750,120),
                            cv2.FONT_HERSHEY_SIMPLEX,3,(255,0,0),10,cv2.LINE_AA) 
                hand.append(label)
            check_thumb = True
        elif result == False:
            if label == "Right":
                cv2.rectangle(img,(25,25),(375,150),(0,255,0),-1)
                cv2.putText(img,"Dislike",(50,120),
                            cv2.FONT_HERSHEY_SIMPLEX,3,(0,0,255),10,cv2.LINE_AA) 
                hand.append(label)
            elif label == "Left":
                cv2.rectangle(img,(900,25),(1250,150),(0,255,0),-1)
                cv2.putText(img,"Dislike",(925,120),
                            cv2.FONT_HERSHEY_SIMPLEX,3,(255,0,0),10,cv2.LINE_AA) 
                hand.append(label)
            check_thumb = True
    if check_thumb:
        return True, img, hand



def check(cv2,np,hands,img,hand):
    for i in hands:
        label = i["label"]
        lms = i["lms"]
        result=None
        num_check = 0
        lms_index = np.array([12,16,20])
        if label == "Right" and "Right" not in hand:   
            for i in lms_index:
                if lms[i].y < lms[i-1].y and lms[i-1].y < lms[i-2].y and lms[i-2].y <lms[i-3].y:
                    num_check+=1
            if num_check==3:
                if lms[4].x > lms[5].x and lms[8].x >lms[5].x:
                    if lms[4].y <  lms[2].y and lms[8].y > lms[6].y:
                        dis = (lms[8].y*100)/lms[4].y
                    else :
                        dis = 0
                    if dis >=90:
                        result = True

        if label == "Left" and "Left" not in hand:
            for i in lms_index:
                if lms[i].y < lms[i-1].y and lms[i-1].y < lms[i-2].y and lms[i-2].y <lms[i-3].y:
                    num_check+=1
            if num_check==3:
                if lms[4].x < lms[5].x and lms[8].x < lms[5].x:
                    if lms[4].y <  lms[2].y and lms[8].y > lms[6].y:
                        dis = (lms[8].y*100)/lms[4].y
                    else :
                        dis = 0
                    if dis >=90:
                        result = True

        if result == True:
            if label == "Right":
                cv2.rectangle(img,(25,25),(200,150),(0,255,0),-1)
                cv2.putText(img,"OK",(50,120),
                                cv2.FONT_HERSHEY_SIMPLEX,3,(0,0,255),10,cv2.LINE_AA)
                hand.append(label)
            elif label == "Left":
                cv2.rectangle(img,(1075,25),(1250,150),(0,255,0),-1)
                cv2.putText(img,"OK",(1100,120),
                                cv2.FONT_HERSHEY_SIMPLEX,3,(255,0,0),10,cv2.LINE_AA)
                hand.append(label)
                
    if result:
        return True,img,hand


import pygesture.handtrack as ht
import cv2,os,sys
from PIL import Image
import numpy as np

currentDir = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(currentDir,"function"))
from .function import number,heart,thumb,power,peace,ok,loveu

cap = cv2.VideoCapture(0)
cap.set(cv2.CAP_PROP_FRAME_WIDTH, 1920)
cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 1080)

ht = ht.handDetector()

stopSign = Image.open(os.path.join(currentDir,"assets","stop.png"))
stopSign = stopSign.resize((200, 200), Image.Resampling.LANCZOS)

heartImg = Image.open(os.path.join(currentDir,"assets","heart.png")).convert("RGBA")
heartImg = heartImg.resize((200, 200), Image.Resampling.LANCZOS)

heartLeftImg = Image.open(os.path.join(currentDir,"assets","heart_left.png")).convert("RGBA")
heartLeftImg = heartLeftImg.resize((250, 250), Image.Resampling.LANCZOS)

heartRightImg = Image.open(os.path.join(currentDir,"assets","heart_right.png")).convert("RGBA")
heartRightImg = heartRightImg.resize((250, 250), Image.Resampling.LANCZOS)


while True:
    CurrentDir = os.path.dirname(os.path.realpath(__file__))
    success, img = cap.read()
    # img = Image.open(os.path.join(CurrentDir,"2hand.jpg"))
    # img = img.convert("RGB")
    # img = np.asarray(img)
    img, hands = ht.findHands(img)
    if hands:
        hand = []
        
        check_loveu = loveu.check(ht,cv2,np,Image,hands,img,hand,stopSign)
        if isinstance(check_loveu,tuple):
            check_loveu, img, hand = check_loveu
            
            
        if len(hand) <= 1:
            check_heart = heart.check(ht,cv2,np,Image,hands,img,hand,heartImg,heartLeftImg,heartRightImg)
            if isinstance(check_heart,tuple):
                check_heart, img, hand = check_heart
                
        if len(hand) <= 1:
            check_thumb = thumb.check(ht,cv2,np,hands,img,hand)
            if isinstance(check_thumb,tuple):
                check_thumb, img, hand = check_thumb
                
        if len(hand) <= 1:
            check_power = power.check(cv2,np,hands,img,hand)
            if isinstance(check_power,tuple):
                check_power, img, hand = check_power

        if len(hand) <= 1:
            check_peace = peace.check(cv2,np,hands,img,hand)
            if isinstance(check_peace,tuple):
                check_peace, img, hand = check_peace
                
        if len(hand) <= 1:
            check_ok = ok.check(cv2,np,hands,img,hand)
            if isinstance(check_ok,tuple):
                check_ok, img, hand = check_ok
        
        if len(hand) <= 1:
            check_number = number.check(cv2,np,hands,img,hand)
            if isinstance(check_number,tuple):
                check_number, img, hand = check_number
        # print(hand)
    cv2.namedWindow("Gesture Recognition", cv2.WND_PROP_FULLSCREEN)
    cv2.setWindowProperty("Gesture Recognition",cv2.WND_PROP_FULLSCREEN,cv2.WINDOW_FULLSCREEN)
    cv2.imshow("Gesture Recognition", img)
    cv2.waitKey(1)


from setuptools import setup, find_packages

with open('./requirement.txt') as f:
  required = f.read().splitlines()

classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Intended Audience :: Information Technology',
  'Operating System :: Unix',
  'Operating System :: MacOS :: MacOS X',
  'Operating System :: Microsoft :: Windows',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3'
]

setup(
  name='pygesture',
  version='1.0.4',
  description='Pre-built OpenCV-Python/Mediapipe modules that easy to use and understand.',
  long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
  long_description_content_type='text/markdown',
  url='https://github.com/GoodDay360/pygesture',  
  author='GoodDay360',
  author_email='istartgame31@gmail.com',
  license='MIT', 
  classifiers=classifiers,
  keywords=['pygesture','library','module','gesture_recognition'], 
  packages=find_packages(exclude=["dist","git","pygesture.egg-info"]),
  install_requires=required,
)


def check(ht,cv2,np,Image,hands,img,hand,stopSign):
    middle = False
    for i in hands:
        label = i["label"]
        lms = i["lms"]
        cv2.putText(img,str(label),(ht.getCoord(lms[0]).x,ht.getCoord(lms[0]).y),
                    cv2.FONT_HERSHEY_SIMPLEX,1,(0,255,0),2,cv2.LINE_AA)
        lms_index = np.array([8,12,16,20])
        result = 0
        for i in lms_index:
            if lms[i].y > lms[i-2].y:
                result += 1
        if result != 4:
            lms_index = np.array([8,16,20])
            result = 0
            for i in lms_index:
                if lms[i].y > lms[i-2].y:
                    result+=1
            if result == 3:
                if lms[12].y < lms[10].y:
                    result+=1
                    img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
                    img = img.copy()
                    if label == "Left" and "Left" not in hand:
                        middle = True
                        hand.append(label)
                        img.paste(stopSign,(ht.getCoord(lms[11]).x-85,ht.getCoord(lms[11]).y),mask=stopSign)
                    elif label == "Right" and "Right" not in hand:
                        middle = True
                        hand.append(label)
                        img.paste(stopSign,(ht.getCoord(lms[11]).x-105,ht.getCoord(lms[11]).y),mask=stopSign)
                    img = cv2.cvtColor(np.array(img), cv2.COLOR_RGB2BGR)
    if middle:
        return True, img, hand
    else:
        result = False
        if result != 4:
            for i in hands:
                label = i["label"]
                lms = i["lms"]
                num_check=0
                if lms[17].x < lms[5].x and lms[4].x > lms[3].x:
                    if lms[16].y > lms[14].y and lms[12].y > lms[9].y:
                        if lms[8].y < lms[6].y:
                            num_check+=1
                        if lms[4].x > lms[1].x:
                            num_check+=1
                        if lms[20].y < lms[17].y:
                            num_check+=1
                if lms[17].x > lms[5].x and lms[4].x < lms[3].x:
                    if lms[16].y > lms[14].y and lms[12].y > lms[9].y:
                        if lms[8].y < lms[6].y:
                            num_check+=1
                        if lms[4].x < lms[1].x:
                            num_check+=1
                        if lms[20].y < lms[17].y:
                            num_check+=1
                if num_check == 3:
                    result = True
                    if label == "Right" and "Right" not in hand:
                        hand.append(label)
                        cv2.rectangle(img,(25,25),(575,150),(0,255,0),-1)
                        cv2.putText(img,"I Love You",(50,120),
                                    cv2.FONT_HERSHEY_SIMPLEX,3,(0,0,255),10,cv2.LINE_AA) 
                    elif label == "Left" and "Left" not in hand:
                        hand.append(label)
                        cv2.rectangle(img,(650,25),(1200,150),(0,255,0),-1)
                        cv2.putText(img,"I Love You",(675,120),
                                    cv2.FONT_HERSHEY_SIMPLEX,3,(255,0,0),10,cv2.LINE_AA) 
            if result:
                return True, img, hand
